@@ -6,9 +6,12 @@ import copy
 import json
 import numbers
 import os
+from typing import Any, Dict, Iterable, List, Optional, Union
+
+ConfigDict = Dict[str, Union[str, int, float]]
 
 
-class SHConfig:
+class SHConfig:  # pylint: disable=too-many-instance-attributes
     """A sentinelhub-py package configuration class.
 
     The class reads during its first initialization the configurable settings from ``./config.json`` file:
@@ -45,145 +48,106 @@ class SHConfig:
 
     """
 
-    class _SHConfig:
-        """Internal class holding configuration parameters"""
+    CREDENTIALS = {
+        "instance_id",
+        "sh_client_id",
+        "sh_client_secret",
+        "aws_access_key_id",
+        "aws_secret_access_key",
+        "aws_session_token",
+    }
+    CONFIG_PARAMS = [
+        "instance_id",
+        "sh_client_id",
+        "sh_client_secret",
+        "sh_base_url",
+        "sh_auth_base_url",
+        "geopedia_wms_url",
+        "geopedia_rest_url",
+        "aws_access_key_id",
+        "aws_secret_access_key",
+        "aws_session_token",
+        "aws_metadata_url",
+        "aws_s3_l1c_bucket",
+        "aws_s3_l2a_bucket",
+        "opensearch_url",
+        "max_wfs_records_per_query",
+        "max_opensearch_records_per_query",
+        "max_download_attempts",
+        "download_sleep_time",
+        "download_timeout_seconds",
+        "number_of_download_processes",
+    ]
 
-        CONFIG_PARAMS = {
-            "instance_id": "",
-            "sh_client_id": "",
-            "sh_client_secret": "",
-            "sh_base_url": "https://services.sentinel-hub.com",
-            "sh_auth_base_url": "https://services.sentinel-hub.com",
-            "geopedia_wms_url": "https://service.geopedia.world",
-            "geopedia_rest_url": "https://www.geopedia.world/rest",
-            "aws_access_key_id": "",
-            "aws_secret_access_key": "",
-            "aws_session_token": "",
-            "aws_metadata_url": "https://roda.sentinel-hub.com",
-            "aws_s3_l1c_bucket": "sentinel-s2-l1c",
-            "aws_s3_l2a_bucket": "sentinel-s2-l2a",
-            "opensearch_url": "http://opensearch.sentinel-hub.com/resto/api/collections/Sentinel2",
-            "max_wfs_records_per_query": 100,
-            "max_opensearch_records_per_query": 500,
-            "max_download_attempts": 4,
-            "download_sleep_time": 5.0,
-            "download_timeout_seconds": 120.0,
-            "number_of_download_processes": 1,
-        }
-        CREDENTIALS = {
-            "instance_id",
-            "sh_client_id",
-            "sh_client_secret",
-            "aws_access_key_id",
-            "aws_secret_access_key",
-            "aws_session_token",
-        }
+    _cache: Optional[Dict[str, Any]] = None
 
-        def __init__(self):
-            self.load_configuration()
-
-        def _parse_configuration(self, config):
-            """Checks if configuration file contains all keys and parses values"""
-            for param, default_value in self.CONFIG_PARAMS.items():
-                if param not in config:
-                    config[param] = default_value
-
-            for param, default_param in self.CONFIG_PARAMS.items():
-                param_type = type(default_param)
-                if (param_type is float) and isinstance(config[param], numbers.Number):
-                    continue
-                if not isinstance(config[param], param_type):
-                    raise ValueError(f"Value of parameter '{param}' must be of type {param_type.__name__}")
-
-            if config["max_wfs_records_per_query"] > 100:
-                raise ValueError("Value of config parameter 'max_wfs_records_per_query' must be at most 100")
-            if config["max_opensearch_records_per_query"] > 500:
-                raise ValueError("Value of config parameter 'max_opensearch_records_per_query' must be at most 500")
-
-            # The following enables that url parameters can be written with or without / at the end
-            for param, value in config.items():
-                if isinstance(value, str) and value.startswith("http"):
-                    config[param] = value.rstrip("/")
-
-            return config
-
-        def get_config_file(self):
-            """Checks if configuration file exists and returns its file path.
-            If file doesn't exist it creates a default configurations file.
-
-            :return: location of configuration file
-            :rtype: str
-            """
-            config_file = os.path.join(os.path.dirname(__file__), "config.json")
-
-            if not os.path.isfile(config_file):
-                with open(config_file, "w") as cfg_file:
-                    json.dump(self.CONFIG_PARAMS, cfg_file, indent=2)
-
-            return config_file
-
-        def load_configuration(self):
-            """Method reads and loads the configuration file."""
-            with open(self.get_config_file(), "r") as cfg_file:
-                config = json.load(cfg_file)
-
-            config = self._parse_configuration(config)
-
-            for prop in config:
-                if prop in self.CONFIG_PARAMS:
-                    setattr(self, prop, config[prop])
-
-        def get_config(self):
-            """Returns a dictionary with configuration parameters
-
-            :return: A dictionary
-            :rtype: dict
-            """
-            config = {item: getattr(self, item) for item in self.CONFIG_PARAMS}
-            for item, default_value in self.CONFIG_PARAMS.items():
-                if config[item] is None:
-                    config[item] = default_value
-            return config
-
-        def save_configuration(self):
-            """Method saves changed parameter values to the configuration file."""
-            config = self.get_config()
-
-            self._parse_configuration(config)
-
-            with open(self.get_config_file(), "w") as cfg_file:
-                json.dump(config, cfg_file, indent=2)
-
-    _instance = None
-
-    def __init__(self, hide_credentials=False):
+    def __init__(self, hide_credentials: bool = False, use_defaults: bool = False):
         """
         :param hide_credentials: If `True` then methods that provide the entire content of the config object will mask
             out all credentials. But credentials could still be accessed directly from config object attributes. The
             default is `False`.
-        :type hide_credentials: bool
+        :param use_defaults: Does not load the configuration file, returns config object with defaults only.
         """
+
+        self.instance_id: str = ""
+        self.sh_client_id: str = ""
+        self.sh_client_secret: str = ""
+        self.sh_base_url: str = "https://services.sentinel-hub.com"
+        self.sh_auth_base_url: str = "https://services.sentinel-hub.com"
+        self.geopedia_wms_url: str = "https://service.geopedia.world"
+        self.geopedia_rest_url: str = "https://www.geopedia.world/rest"
+        self.aws_access_key_id: str = ""
+        self.aws_secret_access_key: str = ""
+        self.aws_session_token: str = ""
+        self.aws_metadata_url: str = "https://roda.sentinel-hub.com"
+        self.aws_s3_l1c_bucket: str = "sentinel-s2-l1c"
+        self.aws_s3_l2a_bucket: str = "sentinel-s2-l2a"
+        self.opensearch_url: str = "http://opensearch.sentinel-hub.com/resto/api/collections/Sentinel2"
+        self.max_wfs_records_per_query: int = 100
+        self.max_opensearch_records_per_query: int = 500  # pylint: disable=invalid-name
+        self.max_download_attempts: int = 4
+        self.download_sleep_time: float = 5.0
+        self.download_timeout_seconds: float = 120.0
+        self.number_of_download_processes: int = 1
+
         self._hide_credentials = hide_credentials
 
-        for prop in self.get_params():
-            setattr(self, prop, getattr(self._global_instance, prop))
+        if not use_defaults:
+            for param, value in self._global_cache.items():
+                setattr(self, param, value)
 
-    def __getitem__(self, name):
+    def _validate_values(self) -> None:
+        """Ensures that the values are aligned with expectations."""
+        default = SHConfig(use_defaults=True)
+        for param in self.CONFIG_PARAMS:
+            value = getattr(self, param)
+            default_value = getattr(default, param)
+            param_type = type(default_value)
+
+            if isinstance(value, str) and value.startswith("http"):
+                value = value.rstrip("/")
+            if (param_type is float) and isinstance(value, numbers.Number):
+                continue
+            if not isinstance(value, param_type):
+                raise ValueError(f"Value of parameter '{param}' must be of type {param_type.__name__}")
+        if self.max_wfs_records_per_query > 100:
+            raise ValueError("Value of config parameter 'max_wfs_records_per_query' must be at most 100")
+        if self.max_opensearch_records_per_query > 500:
+            raise ValueError("Value of config parameter 'max_opensearch_records_per_query' must be at most 500")
+
+    def __getitem__(self, name: str) -> Union[str, int, float]:
         """Config parameters can also be accessed as items."""
-        if name in self.get_params():
+        if name in self.CONFIG_PARAMS:
             return getattr(self, name)
         raise KeyError(f"'{name}' is not a supported config parameter")
 
-    def __dir__(self):
-        return sorted(list(dir(super())) + list(self.get_params()))
-
-    def __str__(self):
+    def __str__(self) -> str:
         """Content of SHConfig in json schema. If `hide_credentials` is set to `True` then credentials will be
         masked.
         """
         return json.dumps(self.get_config_dict(), indent=2)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Representation of SHConfig parameters. If `hide_credentials` is set to `True` then credentials will be
         masked.
         """
@@ -195,135 +159,162 @@ class SHConfig:
         return "\n  ".join(repr_list).strip(",") + "\n)"
 
     @property
-    def _global_instance(self):
+    def _global_cache(self) -> Dict[str, Any]:
         """Uses a class attribute to store a global instance of a class with config parameters."""
-        if SHConfig._instance is None:
-            SHConfig._instance = SHConfig._SHConfig()
-        return SHConfig._instance
+        if SHConfig._cache is None:
+            loaded_instance = SHConfig.load(self.get_config_location())
+            SHConfig._cache = {param: getattr(loaded_instance, param) for param in SHConfig.CONFIG_PARAMS}
+        return SHConfig._cache
 
-    def save(self):
+    @classmethod
+    def load(cls, filename: str) -> "SHConfig":
+        """Method that loads configuration parameters from a file. Does not affect global settings.
+
+        :param filename: Path to file from which to read configuration.
+        """
+        with open(filename, "r") as cfg_file:
+            config_dict = json.load(cfg_file)
+
+        config = cls(use_defaults=True)
+        for param, value in config_dict.items():
+            if param in cls.CONFIG_PARAMS:
+                setattr(config, param, value)
+
+        config._validate_values()
+        return config
+
+    def save(self, filename: Optional[str] = None) -> None:
         """Method that saves configuration parameter changes from instance of SHConfig class to global config class and
         to `config.json` file.
 
+        :param filename: Optional name of file to which to save configuration. If not specified saves to global default.
+
         :Example:
             ``my_config = SHConfig()`` \n
             ``my_config.instance_id = '<new instance id>'`` \n
             ``my_config.save()``
         """
-        is_changed = False
-        for prop in self.get_params():
-            if getattr(self, prop) != getattr(self._global_instance, prop):
-                is_changed = True
-                setattr(self._global_instance, prop, getattr(self, prop))
-        if is_changed:
-            self._global_instance.save_configuration()
+        self._validate_values()
 
-    def copy(self):
+        is_changed = False
+        for param in self.CONFIG_PARAMS:
+            if getattr(self, param) != self._global_cache[param]:
+                is_changed = True
+                self._global_cache[param] = getattr(self, param)  # pylint: disable=unsupported-assignment-operation
+
+        if is_changed:
+            config_dict = {param: getattr(self, param) for param in self.CONFIG_PARAMS}
+            with open(filename or self.get_config_location(), "w") as cfg_file:
+                json.dump(config_dict, cfg_file, indent=2)
+
+    def copy(self) -> "SHConfig":
         """Makes a copy of an instance of `SHConfig`"""
         return copy.copy(self)
 
-    def reset(self, params=...):
+    def reset(self, params: Union[str, Iterable[str], object] = ...) -> None:
         """Resets configuration class to initial values. Use `SHConfig.save()` method in order to save this change.
 
         :param params: Parameters which will be reset. Parameters can be specified with a list of names, e.g.
             ``['instance_id', 'aws_access_key_id', 'aws_secret_access_key']``, or as a single name, e.g.
             ``'sh_base_url'``. By default, all parameters will be reset and default value is ``Ellipsis``.
-        :type params: Ellipsis or list(str) or str
         """
+        default = SHConfig(use_defaults=True)
+
         if params is ...:
             params = self.get_params()
         if isinstance(params, str):
-            self._reset_param(params)
-        elif isinstance(params, (list, tuple)):
+            self._reset_param(params, default)
+        elif isinstance(params, Iterable):
             for param in params:
-                self._reset_param(param)
+                self._reset_param(param, default)
         else:
             raise ValueError(
                 f"Parameters must be specified in form of a list of strings or as a single string, instead got {params}"
             )
 
-    def _reset_param(self, param):
+    def _reset_param(self, param: str, default: "SHConfig") -> None:
         """Resets a single parameter
 
         :param param: A configuration parameter
-        :type param: str
         """
         if param not in self.get_params():
             raise ValueError(f"Cannot reset unknown parameter '{param}'")
-        setattr(self, param, self._SHConfig.CONFIG_PARAMS[param])
+        setattr(self, param, getattr(default, param))
 
-    def get_params(self):
+    def get_params(self) -> List[str]:
         """Returns a list of parameter names
 
         :return: List of parameter names
-        :rtype: list(str)
         """
-        return list(self._SHConfig.CONFIG_PARAMS)
+        return list(self.CONFIG_PARAMS)
 
-    def get_config_dict(self):
+    def get_config_dict(self) -> ConfigDict:
         """Get a dictionary representation of `SHConfig` class. If `hide_credentials` is set to `True` then
         credentials will be masked.
 
         :return: A dictionary with configuration parameters
-        :rtype: dict
         """
-        config_params = {param: getattr(self, param) for param in self.get_params()}
+        config_params = {param: getattr(self, param) for param in self.CONFIG_PARAMS}
 
         if self._hide_credentials:
             config_params = {param: self._mask_credentials(param, value) for param, value in config_params.items()}
 
         return config_params
 
-    def get_config_location(self):
+    @classmethod
+    def get_config_location(cls) -> str:
         """Returns location of configuration file on disk
 
         :return: File path of `config.json` file
-        :rtype: str
         """
-        return self._global_instance.get_config_file()
+        config_file = os.path.join(os.path.dirname(__file__), "config.json")
 
-    def has_eocloud_url(self):
-        """Checks if base Sentinel Hub URL is set to eocloud URL
+        if not os.path.isfile(config_file):
+            with open(config_file, "w") as cfg_file:
+                default_dict = cls(use_defaults=True).get_config_dict()
+                json.dump(default_dict, cfg_file, indent=2)
 
-        :return: `True` if 'eocloud' string is in base OGC URL else `False`
-        :rtype: bool
-        """
-        return "eocloud" in self.sh_base_url
+        return config_file
 
-    def get_sh_oauth_url(self):
+    def _mask_credentials(self, param: str, value: object) -> object:
+        """In case a parameter that holds credentials is given it will mask its value"""
+        if not (param in self.CREDENTIALS and value):
+            return value
+        if not isinstance(value, str):
+            raise ValueError(f"Parameter '{param}' should be a string but {value} found")
+
+        hide_size = min(max(len(value) - 4, 10), len(value))
+        return "*" * hide_size + value[hide_size:]
+
+    def get_sh_oauth_url(self) -> str:
         """Provides URL for Sentinel Hub authentication endpoint
 
         :return: A URL endpoint
-        :rtype: str
         """
         return f"{self.sh_auth_base_url}/oauth/token"
 
-    def get_sh_process_api_url(self):
+    def get_sh_process_api_url(self) -> str:
         """Provides URL for Sentinel Hub Process API endpoint
 
         :return: A URL endpoint
-        :rtype: str
         """
         return f"{self.sh_base_url}/api/v1/process"
 
-    def get_sh_ogc_url(self):
+    def get_sh_ogc_url(self) -> str:
         """Provides URL for Sentinel Hub OGC endpoint
 
         :return: A URL endpoint
-        :rtype: str
         """
-        ogc_endpoint = "v1" if self.has_eocloud_url() else "ogc"
-        return f"{self.sh_base_url}/{ogc_endpoint}"
+        return f"{self.sh_base_url}/ogc"
 
-    def get_sh_rate_limit_url(self):
+    def get_sh_rate_limit_url(self) -> str:
         """Provides URL for Sentinel Hub rate limiting endpoint
 
         :return: A URL endpoint
-        :rtype: str
         """
         return f"{self.sh_auth_base_url}/aux/ratelimit"
 
-    def raise_for_missing_instance_id(self):
+    def raise_for_missing_instance_id(self) -> None:
         """In case Sentinel Hub instance ID is missing it raises an informative error
 
         :raises: ValueError
@@ -334,13 +325,3 @@ class SHConfig:
                 "Either provide it with SHConfig object or save it into config.json configuration file. "
                 "Check https://sentinelhub-py.readthedocs.io/en/latest/configure.html for more info."
             )
-
-    def _mask_credentials(self, param, value):
-        """In case a parameter that holds credentials is given it will mask its value"""
-        if not (param in self._SHConfig.CREDENTIALS and value):
-            return value
-        if not isinstance(value, str):
-            raise ValueError(f"Parameter '{param}' should be a string but {value} found")
-
-        hide_size = min(max(len(value) - 4, 10), len(value))
-        return "*" * hide_size + value[hide_size:]
