@@ -6,4 +6,4 @@ from .client import DownloadClient
 from .request import DownloadRequest
 from .sentinelhub_client import SentinelHubDownloadClient
 from .sentinelhub_statistical_client import SentinelHubStatisticalDownloadClient
-from .session import SentinelHubSession, SessionSharingThread, collect_shared_session
+from .session import SentinelHubSession, SessionSharing, SessionSharingThread, collect_shared_session
